/*
 * Copyright The OpenTelemetry Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
import { ValueType } from '@opentelemetry/api-metrics';
import * as core from '@opentelemetry/core';
import { otlpTypes, toCollectorResource } from '@opentelemetry/exporter-trace-otlp-http';
import { AggregationTemporality, DataPointType, InstrumentType } from '@opentelemetry/sdk-metrics-base';
/**
 * Converts {@link SpanAttributes} to a collector-compatible format.
 * @param attributes
 */
export function toCollectorAttributes(attributes) {
    return Object.entries(attributes).map(function (_a) {
        var _b = __read(_a, 2), key = _b[0], value = _b[1];
        return { key: key, value: String(value) };
    });
}
/**
 * Convert {@link AggregationTemporality} to a collector-compatible format.
 * @param aggregationTemporality
 */
export function toAggregationTemporality(aggregationTemporality) {
    if (aggregationTemporality === AggregationTemporality.CUMULATIVE) {
        return otlpTypes.opentelemetryProto.metrics.v1.AggregationTemporality
            .AGGREGATION_TEMPORALITY_CUMULATIVE;
    }
    if (aggregationTemporality === AggregationTemporality.DELTA) {
        return otlpTypes.opentelemetryProto.metrics.v1.AggregationTemporality
            .AGGREGATION_TEMPORALITY_DELTA;
    }
    return otlpTypes.opentelemetryProto.metrics.v1.AggregationTemporality
        .AGGREGATION_TEMPORALITY_UNSPECIFIED;
}
/**
 * Convert {@link MetricData} of {@link DataPointType.SINGULAR} to a collector-compatible format.
 * @param metricData
 */
export function toSingularDataPoints(metricData) {
    return Array.from(metricData.dataPoints.map(function (dataPoint) {
        return {
            labels: toCollectorAttributes(dataPoint.attributes),
            value: dataPoint.value,
            startTimeUnixNano: core.hrTimeToNanoseconds(dataPoint.startTime),
            timeUnixNano: core.hrTimeToNanoseconds(dataPoint.endTime),
        };
    }));
}
/**
 * Convert {@link MetricData} of {@link DataPointType.HISTOGRAM} to a collector-compatible format.
 * @param metricData
 */
export function toHistogramDataPoints(metricData) {
    return Array.from(metricData.dataPoints.map(function (dataPoints) {
        var histogram = dataPoints.value;
        return {
            labels: toCollectorAttributes(dataPoints.attributes),
            sum: histogram.sum,
            count: histogram.count,
            startTimeUnixNano: core.hrTimeToNanoseconds(dataPoints.startTime),
            timeUnixNano: core.hrTimeToNanoseconds(dataPoints.endTime),
            bucketCounts: histogram.buckets.counts,
            explicitBounds: histogram.buckets.boundaries,
        };
    }));
}
/**
 * Converts {@link MetricData} to a collector-compatible format.
 * @param metricData
 * @param aggregationTemporality
 */
export function toCollectorMetric(metricData, aggregationTemporality) {
    var metricCollector = {
        name: metricData.descriptor.name,
        description: metricData.descriptor.description,
        unit: metricData.descriptor.unit,
    };
    if (metricData.dataPointType === DataPointType.SINGULAR) {
        var result = {
            dataPoints: toSingularDataPoints(metricData),
            isMonotonic: metricData.descriptor.type === InstrumentType.COUNTER ||
                metricData.descriptor.type === InstrumentType.OBSERVABLE_COUNTER,
            aggregationTemporality: toAggregationTemporality(aggregationTemporality),
        };
        if (metricData.descriptor.type === InstrumentType.COUNTER ||
            metricData.descriptor.type === InstrumentType.OBSERVABLE_COUNTER ||
            metricData.descriptor.type === InstrumentType.UP_DOWN_COUNTER ||
            metricData.descriptor.type === InstrumentType.OBSERVABLE_UP_DOWN_COUNTER) {
            if (metricData.descriptor.valueType === ValueType.INT) {
                metricCollector.intSum = result;
            }
            else {
                metricCollector.doubleSum = result;
            }
        }
        else {
            // Instrument is a gauge.
            if (metricData.descriptor.valueType === ValueType.INT) {
                metricCollector.intGauge = result;
            }
            else {
                metricCollector.doubleGauge = result;
            }
        }
    }
    else if (metricData.dataPointType === DataPointType.HISTOGRAM) {
        var result = {
            dataPoints: toHistogramDataPoints(metricData),
            aggregationTemporality: toAggregationTemporality(aggregationTemporality)
        };
        if (metricData.descriptor.valueType === ValueType.INT) {
            metricCollector.intHistogram = result;
        }
        else {
            metricCollector.doubleHistogram = result;
        }
    }
    // TODO: Add support for exponential histograms when they're ready.
    return metricCollector;
}
/**
 * Prepares metric service request to be sent to collector
 * @param metrics metrics
 * @param aggregationTemporality
 * @param collectorExporterBase
 */
export function toOTLPExportMetricServiceRequest(metrics, aggregationTemporality, collectorExporterBase) {
    var additionalAttributes = Object.assign({}, collectorExporterBase.attributes);
    return {
        resourceMetrics: toCollectorResourceMetrics(metrics, additionalAttributes, aggregationTemporality),
    };
}
/**
 * Convert to InstrumentationLibraryMetrics
 * @param instrumentationLibrary
 * @param metrics
 * @param aggregationTemporality
 */
function toCollectorInstrumentationLibraryMetrics(instrumentationLibrary, metrics, aggregationTemporality) {
    return {
        metrics: metrics.map(function (metric) { return toCollectorMetric(metric, aggregationTemporality); }),
        instrumentationLibrary: instrumentationLibrary,
    };
}
/**
 * Returns a list of resource metrics which will be exported to the collector
 * @param resourceMetrics
 * @param baseAttributes
 * @param aggregationTemporality
 */
function toCollectorResourceMetrics(resourceMetrics, baseAttributes, aggregationTemporality) {
    return [{
            resource: toCollectorResource(resourceMetrics.resource, baseAttributes),
            instrumentationLibraryMetrics: Array.from(resourceMetrics.instrumentationLibraryMetrics.map(function (instrumentationLibraryMetrics) { return toCollectorInstrumentationLibraryMetrics(instrumentationLibraryMetrics.instrumentationLibrary, instrumentationLibraryMetrics.metrics, aggregationTemporality); }))
        }];
}
//# sourceMappingURL=transformMetrics.js.map