"use strict";
/*
 * Copyright The OpenTelemetry Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.toOTLPExportMetricServiceRequest = exports.toCollectorMetric = exports.toHistogramDataPoints = exports.toSingularDataPoints = exports.toAggregationTemporality = exports.toCollectorAttributes = void 0;
const api_metrics_1 = require("@opentelemetry/api-metrics");
const core = require("@opentelemetry/core");
const exporter_trace_otlp_http_1 = require("@opentelemetry/exporter-trace-otlp-http");
const sdk_metrics_base_1 = require("@opentelemetry/sdk-metrics-base");
/**
 * Converts {@link SpanAttributes} to a collector-compatible format.
 * @param attributes
 */
function toCollectorAttributes(attributes) {
    return Object.entries(attributes).map(([key, value]) => {
        return { key, value: String(value) };
    });
}
exports.toCollectorAttributes = toCollectorAttributes;
/**
 * Convert {@link AggregationTemporality} to a collector-compatible format.
 * @param aggregationTemporality
 */
function toAggregationTemporality(aggregationTemporality) {
    if (aggregationTemporality === sdk_metrics_base_1.AggregationTemporality.CUMULATIVE) {
        return exporter_trace_otlp_http_1.otlpTypes.opentelemetryProto.metrics.v1.AggregationTemporality
            .AGGREGATION_TEMPORALITY_CUMULATIVE;
    }
    if (aggregationTemporality === sdk_metrics_base_1.AggregationTemporality.DELTA) {
        return exporter_trace_otlp_http_1.otlpTypes.opentelemetryProto.metrics.v1.AggregationTemporality
            .AGGREGATION_TEMPORALITY_DELTA;
    }
    return exporter_trace_otlp_http_1.otlpTypes.opentelemetryProto.metrics.v1.AggregationTemporality
        .AGGREGATION_TEMPORALITY_UNSPECIFIED;
}
exports.toAggregationTemporality = toAggregationTemporality;
/**
 * Convert {@link MetricData} of {@link DataPointType.SINGULAR} to a collector-compatible format.
 * @param metricData
 */
function toSingularDataPoints(metricData) {
    return Array.from(metricData.dataPoints.map(dataPoint => {
        return {
            labels: toCollectorAttributes(dataPoint.attributes),
            value: dataPoint.value,
            startTimeUnixNano: core.hrTimeToNanoseconds(dataPoint.startTime),
            timeUnixNano: core.hrTimeToNanoseconds(dataPoint.endTime),
        };
    }));
}
exports.toSingularDataPoints = toSingularDataPoints;
/**
 * Convert {@link MetricData} of {@link DataPointType.HISTOGRAM} to a collector-compatible format.
 * @param metricData
 */
function toHistogramDataPoints(metricData) {
    return Array.from(metricData.dataPoints.map(dataPoints => {
        const histogram = dataPoints.value;
        return {
            labels: toCollectorAttributes(dataPoints.attributes),
            sum: histogram.sum,
            count: histogram.count,
            startTimeUnixNano: core.hrTimeToNanoseconds(dataPoints.startTime),
            timeUnixNano: core.hrTimeToNanoseconds(dataPoints.endTime),
            bucketCounts: histogram.buckets.counts,
            explicitBounds: histogram.buckets.boundaries,
        };
    }));
}
exports.toHistogramDataPoints = toHistogramDataPoints;
/**
 * Converts {@link MetricData} to a collector-compatible format.
 * @param metricData
 * @param aggregationTemporality
 */
function toCollectorMetric(metricData, aggregationTemporality) {
    const metricCollector = {
        name: metricData.descriptor.name,
        description: metricData.descriptor.description,
        unit: metricData.descriptor.unit,
    };
    if (metricData.dataPointType === sdk_metrics_base_1.DataPointType.SINGULAR) {
        const result = {
            dataPoints: toSingularDataPoints(metricData),
            isMonotonic: metricData.descriptor.type === sdk_metrics_base_1.InstrumentType.COUNTER ||
                metricData.descriptor.type === sdk_metrics_base_1.InstrumentType.OBSERVABLE_COUNTER,
            aggregationTemporality: toAggregationTemporality(aggregationTemporality),
        };
        if (metricData.descriptor.type === sdk_metrics_base_1.InstrumentType.COUNTER ||
            metricData.descriptor.type === sdk_metrics_base_1.InstrumentType.OBSERVABLE_COUNTER ||
            metricData.descriptor.type === sdk_metrics_base_1.InstrumentType.UP_DOWN_COUNTER ||
            metricData.descriptor.type === sdk_metrics_base_1.InstrumentType.OBSERVABLE_UP_DOWN_COUNTER) {
            if (metricData.descriptor.valueType === api_metrics_1.ValueType.INT) {
                metricCollector.intSum = result;
            }
            else {
                metricCollector.doubleSum = result;
            }
        }
        else {
            // Instrument is a gauge.
            if (metricData.descriptor.valueType === api_metrics_1.ValueType.INT) {
                metricCollector.intGauge = result;
            }
            else {
                metricCollector.doubleGauge = result;
            }
        }
    }
    else if (metricData.dataPointType === sdk_metrics_base_1.DataPointType.HISTOGRAM) {
        const result = {
            dataPoints: toHistogramDataPoints(metricData),
            aggregationTemporality: toAggregationTemporality(aggregationTemporality)
        };
        if (metricData.descriptor.valueType === api_metrics_1.ValueType.INT) {
            metricCollector.intHistogram = result;
        }
        else {
            metricCollector.doubleHistogram = result;
        }
    }
    // TODO: Add support for exponential histograms when they're ready.
    return metricCollector;
}
exports.toCollectorMetric = toCollectorMetric;
/**
 * Prepares metric service request to be sent to collector
 * @param metrics metrics
 * @param aggregationTemporality
 * @param collectorExporterBase
 */
function toOTLPExportMetricServiceRequest(metrics, aggregationTemporality, collectorExporterBase) {
    const additionalAttributes = Object.assign({}, collectorExporterBase.attributes);
    return {
        resourceMetrics: toCollectorResourceMetrics(metrics, additionalAttributes, aggregationTemporality),
    };
}
exports.toOTLPExportMetricServiceRequest = toOTLPExportMetricServiceRequest;
/**
 * Convert to InstrumentationLibraryMetrics
 * @param instrumentationLibrary
 * @param metrics
 * @param aggregationTemporality
 */
function toCollectorInstrumentationLibraryMetrics(instrumentationLibrary, metrics, aggregationTemporality) {
    return {
        metrics: metrics.map(metric => toCollectorMetric(metric, aggregationTemporality)),
        instrumentationLibrary,
    };
}
/**
 * Returns a list of resource metrics which will be exported to the collector
 * @param resourceMetrics
 * @param baseAttributes
 * @param aggregationTemporality
 */
function toCollectorResourceMetrics(resourceMetrics, baseAttributes, aggregationTemporality) {
    return [{
            resource: (0, exporter_trace_otlp_http_1.toCollectorResource)(resourceMetrics.resource, baseAttributes),
            instrumentationLibraryMetrics: Array.from(resourceMetrics.instrumentationLibraryMetrics.map(instrumentationLibraryMetrics => toCollectorInstrumentationLibraryMetrics(instrumentationLibraryMetrics.instrumentationLibrary, instrumentationLibraryMetrics.metrics, aggregationTemporality)))
        }];
}
//# sourceMappingURL=transformMetrics.js.map